@@ -34,6 +34,8 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#include <libmissioncontrol/mission-control.h>
+
 #include <libempathy/empathy-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-log-manager.h>
@@ -60,8 +62,6 @@
 
 #define DEBUG_DOMAIN "ChatWindow"
 
-#define URGENCY_TIMEOUT 60*1000
-
 struct _EmpathyChatWindowPriv {
 	EmpathyChatroomManager *chatroom_manager;
 	GList                 *chats;
@@ -73,13 +73,9 @@ struct _EmpathyChatWindowPriv {
 	gboolean               page_added;
 	gboolean               dnd_same_window;
 
-	guint                  urgency_timeout_id;
-
 	GtkWidget             *dialog;
 	GtkWidget             *notebook;
 
-	GtkTooltips           *tooltips;
-
 	/* Menu items. */
 	GtkWidget             *menu_conv_clear;
 	GtkWidget             *menu_conv_insert_smiley;
@@ -281,8 +277,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	priv = GET_PRIV (window);
 
-	priv->tooltips = g_object_ref_sink (gtk_tooltips_new ());
-
 	glade = empathy_glade_get_file ("empathy-chat.glade",
 				       "chat_window",
 				       NULL,
@@ -349,7 +343,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				  window);
 
 	priv->notebook = gtk_notebook_new ();
- 	gtk_notebook_set_group_id (GTK_NOTEBOOK (priv->notebook), 1); 
+ 	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow"); 
 	gtk_box_pack_start (GTK_BOX (chat_vbox), priv->notebook, TRUE, TRUE, 0);
 	gtk_widget_show (priv->notebook);
 
@@ -381,8 +375,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	/* Set up smiley menu */
 	menu = empathy_chat_view_get_smiley_menu (
 		G_CALLBACK (chat_window_insert_smiley_activate_cb),
-		window,
-		priv->tooltips);
+		window);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->menu_conv_insert_smiley), menu);
 
 	/* Set up signals we can't do with glade since we may need to
@@ -510,13 +503,8 @@ empathy_chat_window_finalize (GObject *object)
 		g_source_remove (priv->save_geometry_id);
 	}
 
-	if (priv->urgency_timeout_id != 0) {
-		g_source_remove (priv->urgency_timeout_id);
-	}
-
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
-	g_object_unref (priv->tooltips);
 
 	g_signal_handlers_disconnect_by_func (priv->chatroom_manager,
 					      chat_window_update_menu,
@@ -763,8 +751,8 @@ chat_window_update_menu (EmpathyChatWindow *window)
 		 * connected to the room?
 		 */
 		chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
-							 priv->current_chat->account,
-							 empathy_chat_get_id (priv->current_chat));
+							  empathy_chat_get_account (priv->current_chat),
+							  empathy_chat_get_id (priv->current_chat));
 
 		gtk_widget_set_sensitive (priv->menu_room_add, chatroom == NULL);
 		gtk_widget_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
@@ -895,10 +883,10 @@ chat_window_log_activate_cb (GtkWidget        *menuitem,
 
 	priv = GET_PRIV (window);
 
-	empathy_log_window_show (priv->current_chat->account,
-				empathy_chat_get_id (priv->current_chat),
-				empathy_chat_is_group_chat (priv->current_chat),
-				GTK_WINDOW (priv->dialog));
+	empathy_log_window_show (empathy_chat_get_account (priv->current_chat),
+				 empathy_chat_get_id (priv->current_chat),
+				 empathy_chat_is_group_chat (priv->current_chat),
+				 GTK_WINDOW (priv->dialog));
 }
 
 static void
@@ -914,7 +902,7 @@ chat_window_info_activate_cb (GtkWidget        *menuitem,
 
 	empathy_contact_information_dialog_show (contact,
 						 GTK_WINDOW (priv->dialog),
-						 FALSE);
+						 FALSE, FALSE);
 }
 
 static gboolean
@@ -954,9 +942,9 @@ chat_window_configure_event_cb (GtkWidget         *widget,
 	}
 
 	priv->save_geometry_id =
-		g_timeout_add (500,
-			       (GSourceFunc) chat_window_save_geometry_timeout_cb,
-			       window);
+		g_timeout_add_seconds (1,
+				       (GSourceFunc) chat_window_save_geometry_timeout_cb,
+				       window);
 
 	return FALSE;
 }
@@ -974,7 +962,7 @@ chat_window_conv_activate_cb (GtkWidget         *menuitem,
 
 	manager = empathy_log_manager_new ();
 	log_exists = empathy_log_manager_exists (manager,
-						 priv->current_chat->account,
+						 empathy_chat_get_account (priv->current_chat),
 						 empathy_chat_get_id (priv->current_chat),
 						 empathy_chat_is_group_chat (priv->current_chat));
 	g_object_unref (manager);
@@ -1091,10 +1079,10 @@ chat_window_room_add_activate_cb (GtkWidget        *menuitem,
 		return;
 	}
 
-	chatroom = empathy_chatroom_new_full (priv->current_chat->account,
-					     empathy_chat_get_id (priv->current_chat),
-					     empathy_chat_get_name (priv->current_chat),
-					     FALSE);
+	chatroom = empathy_chatroom_new_full (empathy_chat_get_account (priv->current_chat),
+					      empathy_chat_get_id (priv->current_chat),
+					      empathy_chat_get_name (priv->current_chat),
+					      FALSE);
 
 	manager = empathy_chatroom_manager_new ();
 	empathy_chatroom_manager_add (manager, chatroom);
@@ -1299,10 +1287,7 @@ chat_window_update_tooltip (EmpathyChatWindow *window,
 	}
 
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-tooltip-widget");
-	gtk_tooltips_set_tip (priv->tooltips,
-			      widget,
-			      str,
-			      NULL);
+	gtk_widget_set_tooltip_text (widget, str);
 
 	g_free (str);
 }
@@ -1588,7 +1573,6 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				guint             time,
 				EmpathyChatWindow *window)
 {
-	/* FIXME: DnD of contact do not seems to work... */
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
 		EmpathyContactFactory *factory;
 		EmpathyContact        *contact = NULL;
@@ -1624,8 +1608,22 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		
 		account = empathy_contact_get_account (contact);
 		chat = empathy_chat_window_find_chat (account, id);
-		old_window = empathy_chat_get_window (chat);
-		
+
+		if (!chat) {
+			MissionControl *mc;
+
+			mc = empathy_mission_control_new ();
+			mission_control_request_channel (mc,
+							 empathy_contact_get_account (contact),
+							 TP_IFACE_CHANNEL_TYPE_TEXT,
+							 empathy_contact_get_handle (contact),
+							 TP_HANDLE_TYPE_CONTACT,
+							 NULL, NULL);
+			g_object_unref (mc);
+			return;
+		}
+
+		old_window = empathy_chat_get_window (chat);		
 		if (old_window) {
 			if (old_window == window) {
 				gtk_drag_finish (context, TRUE, FALSE, time);
@@ -1690,21 +1688,6 @@ chat_window_drag_data_received (GtkWidget        *widget,
 	}
 }
 
-static gboolean
-chat_window_urgency_timeout_func (EmpathyChatWindow *window)
-{
-	EmpathyChatWindowPriv *priv;
-
-	priv = GET_PRIV (window);
-
-	empathy_debug (DEBUG_DOMAIN, "Turning off urgency hint");
-	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
-
-	priv->urgency_timeout_id = 0;
-
-	return FALSE;
-}
-
 static void
 chat_window_set_urgency_hint (EmpathyChatWindow *window,
 			      gboolean          urgent)
@@ -1713,29 +1696,9 @@ chat_window_set_urgency_hint (EmpathyChatWindow *window,
 
 	priv = GET_PRIV (window);
 
-	if (!urgent) {
-		/* Remove any existing hint and timeout. */
-		if (priv->urgency_timeout_id) {
-			empathy_debug (DEBUG_DOMAIN, "Turning off urgency hint");
-			gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
-			g_source_remove (priv->urgency_timeout_id);
-			priv->urgency_timeout_id = 0;
-		}
-		return;
-	}
-
-	/* Add a new hint and renew any exising timeout or add a new one. */
-	if (priv->urgency_timeout_id) {
-		g_source_remove (priv->urgency_timeout_id);
-	} else {
-		empathy_debug (DEBUG_DOMAIN, "Turning on urgency hint");
-		gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), TRUE);
-	}
-
-	priv->urgency_timeout_id = g_timeout_add (
-		URGENCY_TIMEOUT,
-		(GSourceFunc) chat_window_urgency_timeout_func,
-		window);
+	empathy_debug (DEBUG_DOMAIN, "Turning %s urgency hint",
+		       urgent ? "on" : "off");
+	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), urgent);
 }
 
 EmpathyChatWindow *
@@ -1918,7 +1881,7 @@ empathy_chat_window_find_chat (McAccount   *account,
 
 			chat = ll->data;
 
-			if (empathy_account_equal (account, chat->account) &&
+			if (empathy_account_equal (account, empathy_chat_get_account (chat)) &&
 			    strcmp (id, empathy_chat_get_id (chat)) == 0) {
 				return chat;
 			}
