@@ -33,10 +33,13 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#include <libmissioncontrol/mission-control.h>
+
 #include <libempathy/empathy-debug.h>
 #include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-tp-contact-list.h>
 #include <libempathy/empathy-contact-factory.h>
+#include <libempathy/empathy-utils.h>
 
 #include "empathy-private-chat.h"
 #include "empathy-chat-view.h"
@@ -78,6 +81,49 @@ static GtkWidget *    private_chat_get_widget                   (EmpathyChat
 
 G_DEFINE_TYPE (EmpathyPrivateChat, empathy_private_chat, EMPATHY_TYPE_CHAT);
 
+
+static GObject *
+private_chat_constructor (GType                  type,
+			  guint                  n_props,
+			  GObjectConstructParam *props)
+{
+	GObject                *chat;
+	EmpathyPrivateChatPriv *priv;
+	EmpathyTpChat          *tp_chat;
+	TpChan                 *tp_chan;
+	McAccount              *account;
+
+	chat = G_OBJECT_CLASS (empathy_private_chat_parent_class)->constructor (type, n_props, props);
+
+	priv = GET_PRIV (chat);
+
+	g_object_get (chat, "tp-chat", &tp_chat, NULL);
+	tp_chan = empathy_tp_chat_get_channel (tp_chat);
+	account = empathy_tp_chat_get_account (tp_chat);
+
+	priv->factory = empathy_contact_factory_new ();
+	priv->contact = empathy_contact_factory_get_from_handle (priv->factory,
+								 account,
+								 tp_chan->handle);
+
+	priv->name = g_strdup (empathy_contact_get_name (priv->contact));
+
+	g_signal_connect (priv->contact, 
+			  "notify::name",
+			  G_CALLBACK (private_chat_contact_updated_cb),
+			  chat);
+	g_signal_connect (priv->contact, 
+			  "notify::presence",
+			  G_CALLBACK (private_chat_contact_presence_updated_cb),
+			  chat);
+
+	priv->is_online = empathy_contact_is_online (priv->contact);
+
+	g_object_unref (tp_chat);
+
+	return chat;
+}
+
 static void
 empathy_private_chat_class_init (EmpathyPrivateChatClass *klass)
 {
@@ -85,6 +131,7 @@ empathy_private_chat_class_init (EmpathyPrivateChatClass *klass)
 	EmpathyChatClass *chat_class = EMPATHY_CHAT_CLASS (klass);
 
 	object_class->finalize = private_chat_finalize;
+	object_class->constructor = private_chat_constructor;
 
 	chat_class->get_name             = private_chat_get_name;
 	chat_class->get_tooltip          = private_chat_get_tooltip;
@@ -98,12 +145,6 @@ empathy_private_chat_class_init (EmpathyPrivateChatClass *klass)
 static void
 empathy_private_chat_init (EmpathyPrivateChat *chat)
 {
-	EmpathyPrivateChatPriv *priv;
-
-	priv = GET_PRIV (chat);
-
-	priv->is_online = FALSE;
-
 	private_chat_create_ui (chat);
 }
 
@@ -171,7 +212,7 @@ private_chat_create_ui (EmpathyPrivateChat *chat)
 
 static void
 private_chat_contact_presence_updated_cb (EmpathyContact     *contact,
-					  GParamSpec        *param,
+					  GParamSpec         *param,
 					  EmpathyPrivateChat *chat)
 {
 	EmpathyPrivateChatPriv *priv;
@@ -206,6 +247,21 @@ private_chat_contact_presence_updated_cb (EmpathyContact     *contact,
 		}
 
 		priv->is_online = TRUE;
+
+		/* If offline message is not supported by CM we need to
+		 * request a new Text Channel. */
+		if (!empathy_chat_is_connected (EMPATHY_CHAT (chat))) {
+			MissionControl *mc;
+
+			mc = empathy_mission_control_new ();
+			mission_control_request_channel (mc,
+							 empathy_contact_get_account (contact),
+							 TP_IFACE_CHANNEL_TYPE_TEXT,
+							 empathy_contact_get_handle (contact),
+							 TP_HANDLE_TYPE_CONTACT,
+							 NULL, NULL);
+			g_object_unref (mc);
+		}
 	}
 
 	g_signal_emit_by_name (chat, "status-changed");
@@ -299,78 +355,16 @@ private_chat_get_widget (EmpathyChat *chat)
 	return priv->widget;
 }
 
-static void
-private_chat_setup (EmpathyPrivateChat *chat,
-		    EmpathyContact     *contact,
-		    EmpathyTpChat     *tp_chat)
-{
-	EmpathyPrivateChatPriv *priv;
-
-	priv = GET_PRIV (chat);
-
-	EMPATHY_CHAT (chat)->account = g_object_ref (empathy_contact_get_account (contact));
-	priv->contact = g_object_ref (contact);
-	priv->name = g_strdup (empathy_contact_get_name (contact));
-
-	empathy_chat_set_tp_chat (EMPATHY_CHAT (chat), tp_chat);
-
-	g_signal_connect (priv->contact, 
-			  "notify::name",
-			  G_CALLBACK (private_chat_contact_updated_cb),
-			  chat);
-	g_signal_connect (priv->contact, 
-			  "notify::presence",
-			  G_CALLBACK (private_chat_contact_presence_updated_cb),
-			  chat);
-
-	priv->is_online = empathy_contact_is_online (priv->contact);
-}
-
 EmpathyPrivateChat *
-empathy_private_chat_new (McAccount *account,
-			  TpChan    *tp_chan)
-{
-	EmpathyPrivateChat     *chat;
-	EmpathyPrivateChatPriv *priv;
-	EmpathyTpChat          *tp_chat;
-	EmpathyContactFactory  *factory;
-	EmpathyContact         *contact;
-
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
-	g_return_val_if_fail (TELEPATHY_IS_CHAN (tp_chan), NULL);
-
-	factory = empathy_contact_factory_new ();
-	contact = empathy_contact_factory_get_from_handle (factory,
-							   account,
-							   tp_chan->handle);
-
-	chat = g_object_new (EMPATHY_TYPE_PRIVATE_CHAT, NULL);
-	priv = GET_PRIV (chat);
-
-	priv->factory = factory;
-	tp_chat = empathy_tp_chat_new (account, tp_chan);
-	private_chat_setup (chat, contact, tp_chat);
-
-	g_object_unref (tp_chat);
-	g_object_unref (contact);
-	g_object_unref (factory);
-
-	return chat;
-}
-
-EmpathyPrivateChat *
-empathy_private_chat_new_with_contact (EmpathyContact *contact)
+empathy_private_chat_new (EmpathyTpChat *tp_chat)
 {
 	EmpathyPrivateChat *chat;
-	EmpathyTpChat      *tp_chat;
 
-	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+	g_return_val_if_fail (EMPATHY_IS_TP_CHAT (tp_chat), NULL);
 
-	chat = g_object_new (EMPATHY_TYPE_PRIVATE_CHAT, NULL);
-	tp_chat = empathy_tp_chat_new_with_contact (contact);
-
-	private_chat_setup (chat, contact, tp_chat);
-	g_object_unref (tp_chat);
+	chat = g_object_new (EMPATHY_TYPE_PRIVATE_CHAT,
+			     "tp-chat", tp_chat,
+			     NULL);
 
 	return chat;
 }
